/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/03/21 16:14:10
 */

import { css, html, Component } from '@bd/core'

class Badge extends Component {
  static props = {
    value: '',
    type: 'info',
    max: {
      type: Number,
      default: null
    },
    hidden: false,
    'is-dot': false
  }

  static styles = [
    css`
      :host {
        position: relative;
        display: inline-block;
      }
      .num {
        z-index: 100;
        position: absolute;
        display: inline-flex;
        justify-content: center;
        align-items: center;
        left: calc(100% - 9px);
        top: -10px;
        height: 18px;
        padding: 0 6px;
        border-radius: 10px;
        font-size: 12px;
        white-space: nowrap;
        color: #fff;
        background: var(--color-blue-3);
      }
      :host([is-dot])::after {
        z-index: 100;
        position: absolute;
        height: 8px;
        width: 8px;
        top: -4px;
        left: calc(100% - 4px);
        background: var(--color-blue-3);
        border-radius: 50%;
        content: '';
      }
      :host([hidden])::after {
        display: none;
        content: '';
      }
    `,

    //配色
    css`
      $colors: (
        primary: 'teal',
        info: 'blue',
        success: 'green',
        warning: 'orange',
        danger: 'red',
        secondary: 'dark',
        help: 'grey'
      );

      @loop $t, $c in $colors {
        :host([type='#{$t}']) {
          .num {
            background-color: var(--color-#{$c}-3);
          }
          &::after {
            background-color: var(--color-#{$c}-3);
          }
        }
      }
    `
  ]
  render() {
    return html`
      <div class="badge" ref="bad">
        <slot></slot>
        ${!this['is-dot'] && !this.hidden
          ? html`<div class="num">
              ${this.max && this.value >= this.max
                ? this.max + '+'
                : this.value}
            </div>`
          : ''}
      </div>
    `
  }
}

Badge.reg('badge')
