/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/03/16 18:05:43
 */
import { css, html, Component, bind, unbind, nextTick } from '@bd/core'
import '../icon/index.js'

class Passwd extends Component {
  static props = {
    type: 'primary',
    icon: '',
    size: 'l',
    value: {
      type: String,
      default: '',
      attribute: false
    },
    _type: {
      type: String,
      default: 'password',
      attribute: false
    },
    autofocus: false,
    readonly: false,
    disabled: false,
    lazy: 0 // 并发拦截时间, 单位毫秒
  }

  static styles = [
    css`
      :host {
        overflow: hidden;
        display: inline-flex;
        min-width: 108px;
        user-select: none;
        -moz-user-select: none;
        color: var(--color-dark-1);
        border-radius: 3px;
        cursor: text;
        transition: box-shadow 0.15s linear;
      }

      .label {
        flex: 1;
        display: flex;
        justify-content: center;
        align-items: center;
        height: 32px;
        font-size: 14px;
        border: 1px solid var(--color-grey-2);
        border-radius: inherit;
        background: var(--input-bg-color, #fff);
        color: inherit;
        cursor: inherit;

        input {
          flex: 1;
          min-width: 36px;
          width: 0;
          height: 100%;
          padding: 0 4px 0 8px;
          border: 0;
          border-radius: inherit;
          color: inherit;
          font: inherit;
          background: none;
          outline: none;
          box-shadow: none;
          cursor: inherit;

          &::placeholder {
            color: var(--color-grey-1);
          }
        }
      }
    `,
    css`
      .label {
        .icon {
          --size: 16px;
          margin: 0 8px 0 4px;
          color: var(--color-grey-2);
          cursor: pointer;
        }
      }
    `,
    // 尺寸
    css`
      @use 'sass:map';
      $sizes: (
        m: (
          w: 72px,
          h: 24px,
          f: 12px
        ),
        l: (
          w: 108px,
          h: 32px,
          f: 14px
        ),
        xl: (
          w: 132px,
          h: 36px,
          f: 14px
        ),
        xxl: (
          w: 160px,
          h: 44px,
          f: 14px
        ),
        xxxl: (
          w: 192px,
          h: 52px,
          f: 16px
        )
      );

      @loop $s, $v in $sizes {
        :host([size='#{$s}']) {
          min-width: map.get($v, 'w');
          .label {
            height: map.get($v, 'h');
          }

          @if $s == 'xxl' {
            .icon {
              --size: 18px;
            }
          }

          @if $s == 'xxxl' {
            .label input {
              padding: 0 6px 0 10px;
            }
            .icon {
              --size: 22px;
              margin: 0 12px 0 4px;
            }
          }
        }
        :host([round][size='#{$s}']) .label input {
          padding: 0 6px 0 20px;
        }
      }
    `,
    // 配色
    css`
      $colors: (
        primary: 'teal',
        info: 'blue',
        success: 'green',
        warning: 'orange',
        danger: 'red',
        secondary: 'dark',
        help: 'grey'
      );

      @loop $t, $c in $colors {
        :host([type='#{$t}']:focus-within) {
          box-shadow: 0 0 0 2px var(--color-#{$c}-a);
        }

        :host([type='#{$t}']) .label {
          border-color: var(--color-#{$c}-2);
        }
      }
    `,

    css`
      :host([disabled]),
      :host([readonly]) {
        cursor: default;
        .icon {
          cursor: default;
        }
      }
      :host([disabled]) {
        cursor: not-allowed;

        .label {
          border-color: var(--color-grey-1);
          background: var(--color-plain-1);
          opacity: 0.6;
        }
      }

      :host([no-border]),
      :host(:focus-within[no-border]) {
        .label {
          border: 0;
        }
      }
    `
  ]

  mounted() {
    if (this.autofocus) {
      nextTick(_ => this.$refs.input.focus())
    }
  }

  iconClick(ev) {
    if (this.readOnly || this.disabled) {
      return
    }
    this._type = this._type === 'password' ? '' : 'password'
  }

  handleInput(ev) {
    this.value = ev.target.value
  }

  handleChange() {
    this.$emit('change')
  }

  render() {
    return html`
      <div class="label">
        <slot class="prepend" name="prepend"></slot>
        <input
          spellcheck="false"
          ref="input"
          @input=${this.handleInput}
          @change=${this.handleChange}
          :readOnly=${this.readOnly}
          :disabled=${this.disabled}
          :type=${this._type}
          :value=${this.value}
        />
        <wc-icon
          class="icon"
          @click=${this.iconClick}
          :name=${this._type === 'password' ? 'eye' : 'eye-off'}
        ></wc-icon>
      </div>
    `
  }
}

Passwd.reg('passwd')
